package io.tswf.minecraft.blockventure.core.network.packet

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.core.api.network.packet.IPlayNetworkPacket
import io.tswf.minecraft.blockventure.core.exception.NetworkPlayPacketSerializationException
import net.minecraft.client.network.ClientPlayNetworkHandler
import net.minecraft.network.PacketByteBuf
import net.minecraft.network.listener.PacketListener
import net.minecraft.network.packet.Packet
import net.minecraft.server.network.ServerPlayNetworkHandler

@Slf4j
@CompileStatic
abstract class NetworkPlayPacketWrapper implements Packet<PacketListener> {

    IPlayNetworkPacket packetObject

    NetworkPlayPacketWrapper(IPlayNetworkPacket packetObject) {
        this.packetObject = packetObject
    }

    NetworkPlayPacketWrapper(IPlayNetworkPacket packetObject, DataInputStream data) {
        this(packetObject)
        try {
            packetObject.readFromBytes(data)
        } catch (Throwable e) {
            throw new NetworkPlayPacketSerializationException(e, packetObject)
        }
        // Релизит буфер майн сам
    }

    @Override
    void write(PacketByteBuf buf) {
        def baos = new ByteArrayOutputStream()
        def data = new DataOutputStream(baos).doThenClose { data ->

            try {
                packetObject.writeToBytes(data)
            } catch (Throwable e) {
                throw new NetworkPlayPacketSerializationException(e, packetObject)
            }

            buf.writeBytes(baos.toByteArray())
        }

        // Релизит буфер майн сам
    }

    @Override
    void apply(PacketListener listener) {
        if (listener instanceof ServerPlayNetworkHandler) {
            packetObject.handleOnServer(listener.player.serverWorld, listener.player)
        } else if (listener instanceof ClientPlayNetworkHandler) {
            packetObject.handleOnClient()
        } else {
            log.warn("There is unknown packet listener '${listener}' found for packet ${packetObject} with type ${packetObject.class.name}! Skipping handle...")
        }
    }
}
